@@ -40,7 +40,7 @@ static gboolean zenity_general_multiple;
 static gboolean zenity_general_editable;
 static gchar   *zenity_general_uri;
 static gboolean zenity_general_dialog_no_wrap;
-static guint    zenity_general_timeout_delay;
+static gint     zenity_general_timeout_delay;
 
 /* Calendar Dialog Options */
 static gboolean zenity_calendar_active;
@@ -61,10 +61,11 @@ static gboolean zenity_error_active;
 static gboolean zenity_info_active;
 
 /* File Selection Dialog Options */
-static gboolean zenity_file_active;
-static gboolean zenity_file_directory;
-static gboolean zenity_file_save;
-static gboolean zenity_file_confirm_overwrite;
+static gboolean       zenity_file_active;
+static gboolean       zenity_file_directory;
+static gboolean       zenity_file_save;
+static gboolean       zenity_file_confirm_overwrite;
+static gchar        **zenity_file_filter;
 
 /* List Dialog Options */
 static gboolean zenity_list_active;
@@ -73,6 +74,7 @@ static gboolean zenity_list_checklist;
 static gboolean zenity_list_radiolist;
 static gchar   *zenity_list_print_column;
 static gchar   *zenity_list_hide_column;
+static gboolean zenity_list_hide_header;
 
 /* Notification Dialog Options */
 static gboolean zenity_notification_active;
@@ -84,9 +86,12 @@ static int      zenity_progress_percentage;
 static gboolean zenity_progress_pulsate;
 static gboolean zenity_progress_auto_close;
 static gboolean zenity_progress_auto_kill;
+static gboolean zenity_progress_no_cancel;
 
 /* Question Dialog Options */
 static gboolean zenity_question_active;
+static gchar   *zenity_question_ok_button;
+static gchar   *zenity_question_cancel_button;
 
 /* Text Dialog Options */
 static gboolean zenity_text_active;
@@ -103,6 +108,15 @@ static gint zenity_scale_step;
 static gboolean zenity_scale_print_partial;
 static gboolean zenity_scale_hide_value;
 
+/* Color Selection Dialog Options */
+static gboolean zenity_colorsel_active;
+static gchar   *zenity_colorsel_color;
+static gboolean zenity_colorsel_show_palette;
+
+/* Password Dialog Options */
+static gboolean zenity_password_active;
+static gboolean zenity_password_show_username;
+
 /* Miscelaneus Options */
 static gboolean zenity_misc_about;
 static gboolean zenity_misc_version;
@@ -151,7 +165,8 @@ static GOptionEntry general_options[] = {
     G_OPTION_ARG_INT,
     &zenity_general_timeout_delay,
     N_("Set dialog timeout in seconds"),
-    NULL
+    /* Timeout for closing the dialog */
+    N_("TIMEOUT")
   },
   {
     NULL
@@ -253,7 +268,7 @@ static GOptionEntry entry_options[] = {
     G_OPTION_ARG_NONE,
     &zenity_entry_hide_text,
     N_("Hide the entry text"),
-    N_("TEXT")
+    NULL
   },
   { 
     NULL 
@@ -391,6 +406,16 @@ static GOptionEntry file_selection_options[] = {
     N_("Confirm file selection if filename already exists"),
     NULL
   },
+  {
+    "file-filter",
+    '\0',
+    0,
+    G_OPTION_ARG_STRING_ARRAY,
+    &zenity_file_filter,
+    N_("Sets a filename filter"),
+    /* Help for file-filter argument (name and patterns for file selection) */
+    N_("NAME | PATTERN1 PATTERN2 ..."),
+  },
   { 
     NULL 
   } 
@@ -488,6 +513,15 @@ static GOptionEntry list_options[] = {
     N_("Hide a specific column"),
     N_("NUMBER")
   },
+  {
+    "hide-header",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_list_hide_header,
+    N_("Hides the column headers"),
+    NULL
+  },
   { 
     NULL 
   } 
@@ -583,6 +617,16 @@ static GOptionEntry progress_options[] = {
     N_("Kill parent process if cancel button is pressed"),
     NULL
   },
+  {
+   "no-cancel",
+   '\0',
+   0,
+   G_OPTION_ARG_NONE,
+   &zenity_progress_no_cancel,
+   /* xgettext: no-c-format */
+   N_("Hide cancel button"),
+   NULL
+  },
   { 
     NULL 
   }
@@ -607,6 +651,24 @@ static GOptionEntry question_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
+  {
+    "ok-label",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_question_ok_button,
+    N_("Sets the label of the Ok button"),
+    N_("TEXT")
+  },
+  {
+    "cancel-label",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_question_cancel_button,
+    N_("Sets the label of the Cancel button"),
+    N_("TEXT")
+  },
   {
     "no-wrap",
     '\0',
@@ -765,6 +827,63 @@ static GOptionEntry scale_options[] = {
   }
 };
 
+static GOptionEntry password_dialog_options[] = {
+  {
+    "password",
+    '\0',
+    G_OPTION_FLAG_IN_MAIN,
+    G_OPTION_ARG_NONE,
+    &zenity_password_active,
+    N_("Display password dialog"),
+    NULL
+  },
+  {
+    "username",
+    '\0',
+    0,
+    G_OPTION_ARG_NONE,
+    &zenity_password_show_username,
+    N_("Display the username option"),
+    NULL
+  },
+  {
+    NULL
+  } 
+};
+
+static GOptionEntry color_selection_options[] = {
+  {
+    "color-selection",
+    '\0',
+    G_OPTION_FLAG_IN_MAIN,
+    G_OPTION_ARG_NONE,
+    &zenity_colorsel_active,
+    N_("Display color selection dialog"),
+    NULL
+  },
+  {
+    "color",
+    '\0',
+    0,
+    G_OPTION_ARG_STRING,
+    &zenity_colorsel_color,
+    N_("Set the color"),
+    N_("VALUE")
+  },
+  {
+    "show-palette",
+    '\0',
+    0,
+    G_OPTION_ARG_NONE,
+    &zenity_colorsel_show_palette,
+    N_("Show the palette"),
+    NULL
+  },
+  {
+    NULL
+  }
+};
+
 static GOptionEntry miscellaneous_options[] = {
   {
     "about",
@@ -809,6 +928,8 @@ zenity_option_init (void) {
   results->text_data = g_new0 (ZenityTextData, 1);
   results->tree_data = g_new0 (ZenityTreeData, 1);
   results->notification_data = g_new0 (ZenityNotificationData, 1);
+  results->color_data = g_new0 (ZenityColorData, 1);
+  results->password_data = g_new0 (ZenityPasswordData, 1);
 }
 
 void
@@ -829,6 +950,9 @@ zenity_option_free (void) {
   if (zenity_entry_entry_text)
     g_free (zenity_entry_entry_text);
 
+  if (zenity_file_filter)
+    g_strfreev (zenity_file_filter);
+
   if (zenity_list_columns)
     g_strfreev (zenity_list_columns);
   if (zenity_list_print_column)
@@ -836,6 +960,14 @@ zenity_option_free (void) {
   if (zenity_list_hide_column)
     g_free (zenity_list_hide_column);
 
+  if (zenity_question_ok_button)
+    g_free (zenity_question_ok_button);
+  if (zenity_question_cancel_button)
+    g_free (zenity_question_cancel_button);
+
+  if (zenity_colorsel_color)
+    g_free (zenity_colorsel_color);
+  
   g_option_context_free (ctx);
 }
 
@@ -955,6 +1087,7 @@ zenity_file_pre_callback (GOptionContext *context,
   zenity_file_directory = FALSE;
   zenity_file_save = FALSE;
   zenity_file_confirm_overwrite = FALSE;
+  zenity_file_filter = NULL;
 
   return TRUE;
 }
@@ -969,6 +1102,7 @@ zenity_list_pre_callback (GOptionContext *context,
   zenity_list_columns = NULL;
   zenity_list_checklist = FALSE;
   zenity_list_radiolist = FALSE;
+  zenity_list_hide_header = FALSE;
   zenity_list_print_column = NULL;
   zenity_list_hide_column = NULL;
 
@@ -998,7 +1132,7 @@ zenity_progress_pre_callback (GOptionContext *context,
   zenity_progress_pulsate = FALSE;
   zenity_progress_auto_close = FALSE;
   zenity_progress_auto_kill = FALSE;
-
+  zenity_progress_no_cancel = FALSE;
   return TRUE;
 }
 
@@ -1052,6 +1186,31 @@ zenity_scale_pre_callback (GOptionContext *context,
   return TRUE;
 }
 
+static gboolean
+zenity_color_pre_callback (GOptionContext *context,
+	                   GOptionGroup   *group,
+		           gpointer	   data,
+		           GError        **error)
+{
+  zenity_colorsel_active = FALSE;
+  zenity_colorsel_color = NULL;
+  zenity_colorsel_show_palette = FALSE;
+
+  return TRUE;
+}
+
+static gboolean
+zenity_password_pre_callback (GOptionContext *context,
+                              GOptionGroup   *group,
+                              gpointer        data,
+                              GError        **error)
+{
+  zenity_password_active = FALSE;
+  zenity_password_show_username = FALSE;
+
+  return TRUE; 
+}
+
 static gboolean
 zenity_misc_pre_callback (GOptionContext *context,
 		          GOptionGroup   *group,
@@ -1207,6 +1366,7 @@ zenity_file_post_callback (GOptionContext *context,
     results->file_data->save = zenity_file_save;
     results->file_data->confirm_overwrite = zenity_file_confirm_overwrite;
     results->file_data->separator = zenity_general_separator;
+    results->file_data->filter = zenity_file_filter;
   } else {
     if (zenity_file_directory)
       zenity_option_error (zenity_option_get_name (file_selection_options, &zenity_file_directory),
@@ -1215,6 +1375,10 @@ zenity_file_post_callback (GOptionContext *context,
     if (zenity_file_save)
       zenity_option_error (zenity_option_get_name (file_selection_options, &zenity_file_save),
                            ERROR_SUPPORT);
+
+    if (zenity_file_filter)
+      zenity_option_error (zenity_option_get_name (file_selection_options, &zenity_file_filter),
+                           ERROR_SUPPORT);
   }
     
   return TRUE;
@@ -1248,6 +1412,7 @@ zenity_list_post_callback (GOptionContext *context,
     results->tree_data->editable = zenity_general_editable;
     results->tree_data->print_column = zenity_list_print_column;
     results->tree_data->hide_column = zenity_list_hide_column;
+    results->tree_data->hide_header = zenity_list_hide_header;
     results->tree_data->separator = zenity_general_separator;
   } else {
     if (zenity_list_columns)
@@ -1269,6 +1434,10 @@ zenity_list_post_callback (GOptionContext *context,
     if (zenity_list_hide_column)
       zenity_option_error (zenity_option_get_name (list_options, &zenity_list_hide_column),
                            ERROR_SUPPORT);
+
+    if (zenity_list_hide_header)
+      zenity_option_error (zenity_option_get_name (list_options, &zenity_list_hide_header),
+                           ERROR_SUPPORT);
   }
     
   return TRUE;
@@ -1308,6 +1477,7 @@ zenity_progress_post_callback (GOptionContext *context,
     results->progress_data->autoclose = zenity_progress_auto_close;
     results->progress_data->autokill = zenity_progress_auto_kill;
     results->progress_data->percentage = zenity_progress_percentage;
+    results->progress_data->no_cancel = zenity_progress_no_cancel;
   } else {
     if (zenity_progress_pulsate)
       zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_pulsate),
@@ -1324,6 +1494,9 @@ zenity_progress_post_callback (GOptionContext *context,
     if (zenity_progress_auto_kill)
       zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_auto_kill),
                            ERROR_SUPPORT);
+    if (zenity_progress_no_cancel)
+      zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_no_cancel),
+			   ERROR_SUPPORT);
   }
     
   return TRUE;
@@ -1341,7 +1514,9 @@ zenity_question_post_callback (GOptionContext *context,
   if (results->mode == MODE_QUESTION) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->mode = ZENITY_MSG_QUESTION;
-    results->msg_data->no_wrap = zenity_general_dialog_no_wrap; 
+    results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
+    results->msg_data->ok_label = zenity_question_ok_button;
+    results->msg_data->cancel_label = zenity_question_cancel_button;
   }
 
   return TRUE;
@@ -1401,6 +1576,48 @@ zenity_scale_post_callback (GOptionContext *context,
   return TRUE;
 }
 
+static gboolean
+zenity_color_post_callback (GOptionContext *context,
+		            GOptionGroup   *group,
+		            gpointer	    data,
+		            GError        **error)
+{
+  zenity_option_set_dialog_mode (zenity_colorsel_active, MODE_COLOR);
+
+  if (results->mode == MODE_COLOR) {
+    results->color_data->color = zenity_colorsel_color;
+    results->color_data->show_palette = zenity_colorsel_show_palette;
+  } else {
+    if (zenity_colorsel_color)
+      zenity_option_error (zenity_option_get_name(color_selection_options, &zenity_colorsel_color),
+                           ERROR_SUPPORT);
+
+    if (zenity_colorsel_show_palette)
+      zenity_option_error (zenity_option_get_name(color_selection_options, &zenity_colorsel_show_palette),
+                           ERROR_SUPPORT);
+  }
+
+  return TRUE;
+}
+
+static gboolean
+zenity_password_post_callback (GOptionContext *context,
+                               GOptionGroup   *group,
+                               gpointer        data,
+                               GError        **error)
+{
+  zenity_option_set_dialog_mode (zenity_password_active, MODE_PASSWORD);
+  if (results->mode == MODE_PASSWORD) {
+    results->password_data->username = zenity_password_show_username;
+  } else {
+    if (zenity_password_show_username)
+      zenity_option_error (zenity_option_get_name(password_dialog_options, &zenity_password_show_username),
+                           ERROR_SUPPORT);
+  }
+
+  return TRUE;
+}
+
 static gboolean
 zenity_misc_post_callback (GOptionContext *context,
 		           GOptionGroup   *group,
@@ -1564,6 +1781,28 @@ zenity_create_context (void)
   g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
   g_option_context_add_group(tmp_ctx, a_group);
   
+  /* Adds color selection option entries */
+  a_group = g_option_group_new("color-selection",
+                               N_("Color selection options"),
+                               N_("Show color selection options"), NULL, NULL);
+  g_option_group_add_entries(a_group, color_selection_options);
+  g_option_group_set_parse_hooks (a_group,
+                    zenity_color_pre_callback, zenity_color_post_callback);
+  g_option_group_set_error_hook (a_group, zenity_option_error_callback);
+  g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
+  g_option_context_add_group(tmp_ctx, a_group);
+
+  /* Adds password dialog option entries */
+  a_group = g_option_group_new("password",
+                               N_("Password dialog options"),
+                               N_("Show password dialog options"), NULL, NULL);
+  g_option_group_add_entries (a_group, password_dialog_options);
+  g_option_group_set_parse_hooks (a_group,
+                    zenity_password_pre_callback, zenity_password_post_callback);
+  g_option_group_set_error_hook (a_group, zenity_option_error_callback);
+  g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
+  g_option_context_add_group(tmp_ctx, a_group);
+
   /* Adds misc option entries */
   a_group = g_option_group_new("misc", 
                                N_("Miscellaneous options"), 
