@@ -98,6 +98,7 @@ typedef struct {
   gboolean autoclose;
   gboolean autokill;
   gdouble  percentage;
+  gboolean no_cancel;
 } ZenityProgressData;
 
 typedef struct {
@@ -125,6 +126,18 @@ typedef struct {
   gboolean listen;
 } ZenityNotificationData;
 
+typedef struct {
+  gchar   *color;
+  gboolean show_palette;
+} ZenityColorData;
+
+typedef struct {
+  gboolean username;
+  gchar *password;
+  GtkWidget *entry_username;
+  GtkWidget *entry_password;
+} ZenityPasswordData;
+
 void    zenity_calendar         (ZenityData             *data,
                                 ZenityCalendarData      *calendar_data);
 void    zenity_msg              (ZenityData             *data,
@@ -141,10 +154,15 @@ void    zenity_tree             (ZenityData             *data,
                                  ZenityTreeData         *tree_data);
 void	zenity_notification	(ZenityData		*data,
 				 ZenityNotificationData	*notification_data);
+void    zenity_colorselection   (ZenityData             *data,
+                                 ZenityColorData        *notification_data);
 void	zenity_scale		(ZenityData		*data,
 				 ZenityScaleData	*scale_data);
 void    zenity_about            (ZenityData             *data);
 
+void    zenity_password_dialog  (ZenityData             *data,
+                                 ZenityPasswordData     *password_data);
+
 G_END_DECLS
 
 #endif /* ZENITY_H */
